@@ -67,9 +67,12 @@ class AutoViewFieldMixin(object):
         "See :py:meth:`.views.Select2View.get_results`."
         raise NotImplementedError
 
+
 import copy
 
 from django import forms
+from django.core import validators
+from django.core.exceptions import ValidationError
 from django.forms.models import ModelChoiceIterator
 from django.db.models import Q
 from django.utils.translation import ugettext_lazy as _
@@ -90,12 +93,14 @@ class Select2ChoiceField(forms.ChoiceField):
     """
     widget = Select2Widget
 
+
 class Select2MultipleChoiceField(forms.MultipleChoiceField):
     """
     Drop-in Select2 replacement for :py:class:`forms.MultipleChoiceField`.
     """
     widget = Select2MultipleWidget
 
+
 ### Model fields related mixins ###
 
 class ModelResultJsonMixin(object):
@@ -235,6 +240,7 @@ class ModelResultJsonMixin(object):
         res = [ (getattr(obj, self.to_field_name), self.label_from_instance(obj), ) for obj in res ]
         return (NO_ERR_RESP, has_more, res, )
 
+
 class UnhideableQuerysetType(type):
     """
     This does some pretty nasty hacky stuff, to make sure users can
@@ -263,6 +269,7 @@ class UnhideableQuerysetType(type):
             kwargs['queryset'] = getattr(cls, '_subclass_queryset')
         return type.__call__(cls, *args, **kwargs)
 
+
 class ChoiceMixin(object):
     """
     Simple mixin which provides a property -- ``choices``. When ``choices`` is set,
@@ -271,6 +278,7 @@ class ChoiceMixin(object):
     def _get_choices(self):
         if hasattr(self, '_choices'):
             return self._choices
+        return []
 
     def _set_choices(self, value):
         # Setting choices also sets the choices on the widget.
@@ -280,6 +288,12 @@ class ChoiceMixin(object):
 
     choices = property(_get_choices, _set_choices)
 
+    def __deepcopy__(self, memo):
+        result = super(ChoiceMixin, self).__deepcopy__(memo)
+        result._choices = copy.deepcopy(self._choices, memo)
+        return result
+
+
 class QuerysetChoiceMixin(ChoiceMixin):
     """
     Overrides ``choices``' getter to return instance of :py:class:`.ModelChoiceIterator`
@@ -303,6 +317,7 @@ class QuerysetChoiceMixin(ChoiceMixin):
 
     choices = property(_get_choices, ChoiceMixin._set_choices)
 
+
 class ModelChoiceFieldMixin(object):
 
     def __init__(self, *args, **kwargs):
@@ -326,14 +341,17 @@ class ModelChoiceFieldMixin(object):
         if hasattr(self, '_queryset'):
             return self._queryset
 
+
 ### Slightly altered versions of the Django counterparts with the same name in forms module. ###
 
 class ModelChoiceField(ModelChoiceFieldMixin, forms.ModelChoiceField):
     queryset = property(ModelChoiceFieldMixin._get_queryset, forms.ModelChoiceField._set_queryset)
 
+
 class ModelMultipleChoiceField(ModelChoiceFieldMixin, forms.ModelMultipleChoiceField):
     queryset = property(ModelChoiceFieldMixin._get_queryset, forms.ModelMultipleChoiceField._set_queryset)
 
+
 ### Light Fileds specialized for Models ###
 
 class ModelSelect2Field(ModelChoiceField) :
@@ -344,6 +362,7 @@ class ModelSelect2Field(ModelChoiceField) :
     """
     widget = Select2Widget
 
+
 class ModelSelect2MultipleField(ModelMultipleChoiceField) :
     """
     Light multiple-value Select2 field, specialized for Models.
@@ -352,11 +371,19 @@ class ModelSelect2MultipleField(ModelMultipleChoiceField) :
     """
     widget = Select2MultipleWidget
 
+
 ### Heavy fields ###
 
-class HeavySelect2FieldBase(ChoiceMixin, forms.Field):
+class HeavySelect2FieldBaseMixin(object):
     """
-    Base field for all Heavy fields.
+    Base mixin field for all Heavy fields.
+
+    .. note:: Although Heavy fields accept ``choices`` parameter like all Django choice fields, but these
+        fields are backed by big data sources, so ``choices`` cannot possibly have all the values.
+
+        For Heavies, consider ``choices`` to be a subset of all possible choices. It is available because users
+        might expect it to be available.
+
     """
     def __init__(self, *args, **kwargs):
         """
@@ -382,7 +409,7 @@ class HeavySelect2FieldBase(ChoiceMixin, forms.Field):
             raise ValueError('data_view is required else you need to provide your own widget instance.')
 
         kargs.update(kwargs)
-        super(HeavySelect2FieldBase, self).__init__(*args, **kargs)
+        super(HeavySelect2FieldBaseMixin, self).__init__(*args, **kargs)
 
         # By this time self.widget would have been instantiated.
 
@@ -398,28 +425,149 @@ class HeavySelect2FieldBase(ChoiceMixin, forms.Field):
             choices = self.choices
         self.choices = choices
 
-class HeavySelect2ChoiceField(HeavySelect2FieldBase):
+
+class HeavyChoiceField(ChoiceMixin, forms.Field):
+    """
+    Reimplements :py:class:`django.forms.TypedChoiceField` in a way which suites the use of big data.
+
+    .. note:: Although this field accepts ``choices`` parameter like all Django choice fields, but these
+        fields are backed by big data sources, so ``choices`` cannot possibly have all the values. It is meant
+        to be a subset of all possible choices.
+    """
+    default_error_messages = {
+        'invalid_choice': _(u'Select a valid choice. %(value)s is not one of the available choices.'),
+    }
+    empty_value = u''
+    "Sub-classes can set this other value if needed."
+
+    def __init__(self, *args, **kwargs):
+        super(HeavyChoiceField, self).__init__(*args, **kwargs)
+        # Widget should have been instantiated by now.
+        self.widget.field = self
+
+    def to_python(self, value):
+        if value == self.empty_value or value in validators.EMPTY_VALUES:
+            return self.empty_value
+        try:
+            value = self.coerce_value(value)
+        except (ValueError, TypeError, ValidationError):
+            raise ValidationError(self.error_messages['invalid_choice'] % {'value': value})
+        return value
+
+    def validate(self, value):
+        super(HeavyChoiceField, self).validate(value)
+        if value and not self.valid_value(value):
+            raise ValidationError(self.error_messages['invalid_choice'] % {'value': value})
+
+    def valid_value(self, value):
+        uvalue = smart_unicode(value)
+        for k, v in self.choices:
+            if uvalue == smart_unicode(k):
+                return True
+        return self.validate_value(value)
+
+    def coerce_value(self, value):
+        """
+        Coerces ``value`` to a Python data type.
+
+        Sub-classes should override this if they do not want unicode values.
+        """
+        return smart_unicode(value)
+
+    def validate_value(self, value):
+        """
+        Sub-classes can override this to validate the value entered against the big data.
+
+        :param value: Value entered by the user.
+        :type value: As coerced by :py:meth:`.coerce_value`.
+
+        :return: ``True`` means the ``value`` is valid.
+        """
+        return True
+
+    def _get_val_txt(self, value):
+        try:
+            value = self.coerce_value(value)
+            self.validate_value(value)
+        except Exception, e:
+            logger.exception("Exception while trying to get label for value")
+            return None
+        return self.get_val_txt(value)
+
+    def get_val_txt(self, value):
+        """
+        If Heavy widgets encounter any value which it can't find in ``choices`` then it calls
+        this method to get the label for the value.
+
+        :param value: Value entered by the user.
+        :type value: As coerced by :py:meth:`.coerce_value`.
+
+        :return: The label for this value.
+        :rtype: :py:obj:`unicode` or None (when no possible label could be found)
+        """
+        return None
+
+class HeavyMultipleChoiceField(HeavyChoiceField):
+    """
+    Reimplements :py:class:`django.forms.TypedMultipleChoiceField` in a way which suites the use of big data.
+
+    .. note:: Although this field accepts ``choices`` parameter like all Django choice fields, but these
+        fields are backed by big data sources, so ``choices`` cannot possibly have all the values. It is meant
+        to be a subset of all possible choices.
+    """
+    hidden_widget = forms.MultipleHiddenInput
+    default_error_messages = {
+        'invalid_choice': _(u'Select a valid choice. %(value)s is not one of the available choices.'),
+        'invalid_list': _(u'Enter a list of values.'),
+    }
+
+    def to_python(self, value):
+        if not value:
+            return []
+        elif not isinstance(value, (list, tuple)):
+            raise ValidationError(self.error_messages['invalid_list'])
+        return [self.coerce_value(val) for val in value]
+
+    def validate(self, value):
+        if self.required and not value:
+            raise ValidationError(self.error_messages['required'])
+        # Validate that each value in the value list is in self.choices or
+        # the big data (i.e. validate_value() returns True).
+        for val in value:
+            if not self.valid_value(val):
+                raise ValidationError(self.error_messages['invalid_choice'] % {'value': val})
+
+
+class HeavySelect2ChoiceField(HeavySelect2FieldBaseMixin, HeavyChoiceField):
     "Heavy Select2 Choice field."
     widget = HeavySelect2Widget
 
-class HeavySelect2MultipleChoiceField(HeavySelect2FieldBase):
+
+class HeavySelect2MultipleChoiceField(HeavySelect2FieldBaseMixin, HeavyMultipleChoiceField):
     "Heavy Select2 Multiple Choice field."
     widget = HeavySelect2MultipleWidget
 
+
 ### Heavy field specialized for Models ###
 
-class HeavyModelSelect2ChoiceField(QuerysetChoiceMixin, HeavySelect2ChoiceField, ModelChoiceField):
+class HeavyModelSelect2ChoiceField(HeavySelect2FieldBaseMixin, ModelChoiceField):
     "Heavy Select2 Choice field, specialized for Models."
+    widget = HeavySelect2Widget
+
     def __init__(self, *args, **kwargs):
         kwargs.pop('choices', None)
         super(HeavyModelSelect2ChoiceField, self).__init__(*args, **kwargs)
 
-class HeavyModelSelect2MultipleChoiceField(QuerysetChoiceMixin, HeavySelect2MultipleChoiceField, ModelMultipleChoiceField):
+
+class HeavyModelSelect2MultipleChoiceField(HeavySelect2FieldBaseMixin, ModelMultipleChoiceField):
     "Heavy Select2 Multiple Choice field, specialized for Models."
+    widget = HeavySelect2MultipleWidget
+
     def __init__(self, *args, **kwargs):
         kwargs.pop('choices', None)
         super(HeavyModelSelect2MultipleChoiceField, self).__init__(*args, **kwargs)
 
+
 ### Heavy general field that uses central AutoView ###
 
 class AutoSelect2Field(AutoViewFieldMixin, HeavySelect2ChoiceField):
@@ -428,6 +576,8 @@ class AutoSelect2Field(AutoViewFieldMixin, HeavySelect2ChoiceField):
 
     This needs to be subclassed. The first instance of a class (sub-class) is used to serve all incoming
     json query requests for that type (class).
+
+    .. warning:: :py:exc:`NotImplementedError` would be thrown if :py:meth:`get_results` is not implemented.
     """
 
     widget = AutoHeavySelect2Widget
@@ -437,12 +587,15 @@ class AutoSelect2Field(AutoViewFieldMixin, HeavySelect2ChoiceField):
         kwargs['data_view'] = self.data_view
         super(AutoSelect2Field, self).__init__(*args, **kwargs)
 
+
 class AutoSelect2MultipleField(AutoViewFieldMixin, HeavySelect2MultipleChoiceField):
     """
     Auto Heavy Select2 field for multiple choices.
 
     This needs to be subclassed. The first instance of a class (sub-class) is used to serve all incoming
     json query requests for that type (class).
+
+    .. warning:: :py:exc:`NotImplementedError` would be thrown if :py:meth:`get_results` is not implemented.
     """
 
     widget = AutoHeavySelect2MultipleWidget
@@ -452,6 +605,7 @@ class AutoSelect2MultipleField(AutoViewFieldMixin, HeavySelect2MultipleChoiceFie
         kwargs['data_view'] = self.data_view
         super(AutoSelect2MultipleField, self).__init__(*args, **kwargs)
 
+
 ### Heavy field, specialized for Model, that uses central AutoView ###
 
 class AutoModelSelect2Field(ModelResultJsonMixin, AutoViewFieldMixin, HeavyModelSelect2ChoiceField):
@@ -471,6 +625,7 @@ class AutoModelSelect2Field(ModelResultJsonMixin, AutoViewFieldMixin, HeavyModel
         kwargs['data_view'] = self.data_view
         super(AutoModelSelect2Field, self).__init__(*args, **kwargs)
 
+
 class AutoModelSelect2MultipleField(ModelResultJsonMixin, AutoViewFieldMixin, HeavyModelSelect2MultipleChoiceField):
     """
     Auto Heavy Select2 field for multiple choices, specialized for Models.
