@@ -16,6 +16,14 @@ from .util import render_js_script, convert_to_js_string_arr, JSVar, JSFunction,
 
 logger = logging.getLogger(__name__)
 
+def get_select2_js_path():
+    from django.conf import settings
+    if settings.configured and settings.DEBUG:
+        return 'js/select2.js'
+    else:
+        return 'js/select2.min.js'
+
+
 ### Light mixin and widgets ###
 
 class Select2Mixin(object):
@@ -116,6 +124,7 @@ class Select2Mixin(object):
     def set_placeholder(self, val):
         """
         Placeholder is a value which Select2 JS library shows when nothing is selected. This should be string.
+        
         :return: None
         """
         self.options['placeholder'] = val
@@ -123,6 +132,7 @@ class Select2Mixin(object):
     def get_options(self):
         """
         :return: Dictionary of options to be passed to Select2 JS.
+        
         :rtype: :py:obj:`dict`
         """
         options = dict(self.options)
@@ -133,6 +143,7 @@ class Select2Mixin(object):
     def render_select2_options_code(self, options, id_):
         """
         Renders options for Select2 JS.
+        
         :return: The rendered JS code.
         :rtype: :py:obj:`unicode`
         """
@@ -141,6 +152,7 @@ class Select2Mixin(object):
     def render_js_code(self, id_, *args):
         """
         Renders the ``<script>`` block which contains the JS code for this widget.
+        
         :return: The rendered JS code enclosed inside ``<script>`` block.
         :rtype: :py:obj:`unicode`
         """
@@ -151,6 +163,7 @@ class Select2Mixin(object):
     def render_inner_js_code(self, id_, *args):
         """
         Renders all the JS code required for this widget.
+        
         :return: The rendered JS code which will be later enclosed inside ``<script>`` block.
         :rtype: :py:obj:`unicode`
         """
@@ -162,6 +175,7 @@ class Select2Mixin(object):
     def render(self, name, value, attrs=None, choices=()):
         """
         Renders this widget. Html and JS code blocks all are rendered by this.
+        
         :return: The rendered markup.
         :rtype: :py:obj:`unicode`
         """
@@ -180,7 +194,7 @@ class Select2Mixin(object):
         return mark_safe(s)
 
     class Media:
-        js = ('js/select2.min.js', )
+        js = (get_select2_js_path(), )
         css = {'screen': ('css/select2.css', 'css/extra.css', )}
 
 
@@ -316,6 +330,7 @@ class HeavySelect2Mixin(Select2Mixin):
                 ``heavy_data.js`` stores the selected values and their labels in the cookies. These are cleared
                 when browser is closed.
         """
+        self.field = None
         self.options = dict(self.options) # Making an instance specific copy
         self.view = kwargs.pop('data_view', None)
         self.url = kwargs.pop('data_url', None)
@@ -354,7 +369,13 @@ class HeavySelect2Mixin(Select2Mixin):
         for val, txt in chain(self.choices, all_choices):
             val = force_unicode(val)
             if val in selected_choices:
+                selected_choices = [v for v in selected_choices if v != val]
                 txts.append(txt)
+        if hasattr(self.field, '_get_val_txt') and selected_choices:
+            for val in selected_choices:
+                txt = self.field._get_val_txt(val)
+                if txt is not None:
+                    txts.append(txt)
         if txts:
             return convert_to_js_string_arr(txts)
 
@@ -399,7 +420,7 @@ class HeavySelect2Mixin(Select2Mixin):
         return js
 
     class Media:
-        js = ('js/select2.min.js', 'js/heavy_data.js', )
+        js = (get_select2_js_path(), 'js/heavy_data.js', )
         css = {'screen': ('css/select2.css', 'css/extra.css', )}
 
 
